@@ -3,6 +3,8 @@ use crate::VaultError;
 use arrayref::array_ref;
 use cfg_if::cfg_if;
 use ockam_core::compat::rand::{thread_rng, RngCore};
+use ockam_core::Result;
+use ockam_core::{async_trait, compat::boxed::Box};
 use ockam_vault_core::{
     KeyId, KeyIdVault, PublicKey, Secret, SecretAttributes, SecretKey, SecretPersistence,
     SecretType, SecretVault, AES128_SECRET_LENGTH, AES256_SECRET_LENGTH, CURVE25519_SECRET_LENGTH,
@@ -13,15 +15,14 @@ cfg_if! {
         use signature_bbs_plus::SecretKey as BlsSecretKey;
     }
 }
-use zeroize::Zeroize;
 
 impl SoftwareVault {
     /// Compute key id from secret and attributes. Only Curve25519 and Buffer types are supported
-    fn compute_key_id(
+    async fn compute_key_id(
         &mut self,
         secret: &[u8],
         attributes: &SecretAttributes,
-    ) -> ockam_core::Result<Option<KeyId>> {
+    ) -> Result<Option<KeyId>> {
         Ok(match attributes.stype() {
             SecretType::Curve25519 => {
                 let sk = x25519_dalek::StaticSecret::from(*array_ref![
@@ -31,9 +32,8 @@ impl SoftwareVault {
                 ]);
                 let public = x25519_dalek::PublicKey::from(&sk);
                 Some(
-                    self.compute_key_id_for_public_key(&PublicKey::new(
-                        public.as_bytes().to_vec(),
-                    ))?,
+                    self.compute_key_id_for_public_key(&PublicKey::new(public.as_bytes().to_vec()))
+                        .await?,
                 )
             }
             #[cfg(feature = "bls")]
@@ -43,18 +43,14 @@ impl SoftwareVault {
                 let bls_secret_key = BlsSecretKey::from_bytes(secret.try_into().unwrap()).unwrap();
                 let public_key =
                     PublicKey::new(BlsPublicKey::from(&bls_secret_key).to_bytes().into());
-                Some(self.compute_key_id_for_public_key(&public_key)?)
+                Some(self.compute_key_id_for_public_key(&public_key).await?)
             }
             SecretType::Buffer | SecretType::Aes | SecretType::P256 => None,
         })
     }
 
     /// Validate secret key.
-    pub fn check_secret(
-        &mut self,
-        secret: &[u8],
-        attributes: &SecretAttributes,
-    ) -> ockam_core::Result<()> {
+    pub fn check_secret(&mut self, secret: &[u8], attributes: &SecretAttributes) -> Result<()> {
         match attributes.stype() {
             #[cfg(feature = "bls")]
             SecretType::Bls => {
@@ -76,14 +72,18 @@ impl SoftwareVault {
     }
 }
 
+#[async_trait]
 impl SecretVault for SoftwareVault {
     /// Generate fresh secret. Only Curve25519 and Buffer types are supported
-    fn secret_generate(&mut self, attributes: SecretAttributes) -> ockam_core::Result<Secret> {
-        let mut rng = thread_rng(); // FIXME
+    async fn secret_generate(&mut self, attributes: SecretAttributes) -> Result<Secret> {
         let key = match attributes.stype() {
             SecretType::Curve25519 => {
-                let mut bytes = vec![0u8; 32];
-                rng.fill_bytes(&mut bytes);
+                let bytes = {
+                    let mut rng = thread_rng();
+                    let mut bytes = vec![0u8; 32];
+                    rng.fill_bytes(&mut bytes);
+                    bytes
+                };
 
                 SecretKey::new(bytes)
             }
@@ -91,8 +91,13 @@ impl SecretVault for SoftwareVault {
                 if attributes.persistence() != SecretPersistence::Ephemeral {
                     return Err(VaultError::InvalidKeyType.into());
                 };
-                let mut key = vec![0u8; attributes.length()];
-                rng.fill_bytes(key.as_mut_slice());
+                let key = {
+                    let mut rng = thread_rng();
+                    let mut key = vec![0u8; attributes.length()];
+                    rng.fill_bytes(key.as_mut_slice());
+                    key
+                };
+
                 SecretKey::new(key)
             }
             SecretType::Aes => {
@@ -104,8 +109,13 @@ impl SecretVault for SoftwareVault {
                 if attributes.persistence() != SecretPersistence::Ephemeral {
                     return Err(VaultError::InvalidKeyType.into());
                 };
-                let mut key = vec![0u8; attributes.length()];
-                rng.fill_bytes(&mut key);
+                let key = {
+                    let mut rng = thread_rng();
+                    let mut key = vec![0u8; attributes.length()];
+                    rng.fill_bytes(key.as_mut_slice());
+                    key
+                };
+
                 SecretKey::new(key)
             }
             SecretType::P256 => {
@@ -113,12 +123,13 @@ impl SecretVault for SoftwareVault {
             }
             #[cfg(feature = "bls")]
             SecretType::Bls => {
+                let mut rng = thread_rng();
                 let bls_secret_key = BlsSecretKey::random(&mut rng).unwrap();
 
                 SecretKey::new(bls_secret_key.to_bytes().to_vec())
             }
         };
-        let key_id = self.compute_key_id(key.as_ref(), &attributes)?;
+        let key_id = self.compute_key_id(key.as_ref(), &attributes).await?;
         self.next_id += 1;
         self.entries
             .insert(self.next_id, VaultEntry::new(key_id, attributes, key));
@@ -126,13 +137,13 @@ impl SecretVault for SoftwareVault {
         Ok(Secret::new(self.next_id))
     }
 
-    fn secret_import(
+    async fn secret_import(
         &mut self,
         secret: &[u8],
         attributes: SecretAttributes,
-    ) -> ockam_core::Result<Secret> {
+    ) -> Result<Secret> {
         self.check_secret(secret, &attributes)?;
-        let key_id_opt = self.compute_key_id(secret, &attributes)?;
+        let key_id_opt = self.compute_key_id(secret, &attributes).await?;
         self.next_id += 1;
         self.entries.insert(
             self.next_id,
@@ -141,16 +152,16 @@ impl SecretVault for SoftwareVault {
         Ok(Secret::new(self.next_id))
     }
 
-    fn secret_export(&mut self, context: &Secret) -> ockam_core::Result<SecretKey> {
+    async fn secret_export(&mut self, context: &Secret) -> Result<SecretKey> {
         self.get_entry(context).map(|i| i.key().clone())
     }
 
-    fn secret_attributes_get(&mut self, context: &Secret) -> ockam_core::Result<SecretAttributes> {
+    async fn secret_attributes_get(&mut self, context: &Secret) -> Result<SecretAttributes> {
         self.get_entry(context).map(|i| i.key_attributes())
     }
 
     /// Extract public key from secret. Only Curve25519 type is supported
-    fn secret_public_key_get(&mut self, context: &Secret) -> ockam_core::Result<PublicKey> {
+    async fn secret_public_key_get(&mut self, context: &Secret) -> Result<PublicKey> {
         let entry = self.get_entry(context)?;
 
         if entry.key().as_ref().len() != CURVE25519_SECRET_LENGTH {
@@ -184,11 +195,11 @@ impl SecretVault for SoftwareVault {
     }
 
     /// Remove secret from memory
-    fn secret_destroy(&mut self, context: Secret) -> ockam_core::Result<()> {
-        if let Some(mut k) = self.entries.remove(&context.index()) {
-            k.zeroize();
+    async fn secret_destroy(&mut self, context: Secret) -> Result<()> {
+        match self.entries.remove(&context.index()) {
+            None => Err(VaultError::EntryNotFound.into()),
+            Some(_) => Ok(()),
         }
-        Ok(())
     }
 }
 
@@ -241,10 +252,13 @@ mod tests {
         }
     }
 
-    fn check_key_id_computation(mut vault: SoftwareVault, sec_idx: Secret) {
-        let public_key = vault.secret_public_key_get(&sec_idx).unwrap();
-        let key_id = vault.compute_key_id_for_public_key(&public_key).unwrap();
-        let sec_idx_2 = vault.get_secret_by_key_id(&key_id).unwrap();
+    async fn check_key_id_computation(mut vault: SoftwareVault, sec_idx: Secret) {
+        let public_key = vault.secret_public_key_get(&sec_idx).await.unwrap();
+        let key_id = vault
+            .compute_key_id_for_public_key(&public_key)
+            .await
+            .unwrap();
+        let sec_idx_2 = vault.get_secret_by_key_id(&key_id).await.unwrap();
         assert_eq!(sec_idx, sec_idx_2)
     }
 
@@ -257,38 +271,41 @@ mod tests {
             .collect()
     }
 
-    #[test]
-    fn secret_generate_compute_key_id() {
+    #[tokio::test]
+    async fn secret_generate_compute_key_id() {
         for attrs in flat_map_options(vec![new_curve255519_attrs(), new_bls_attrs()]) {
             let mut vault = new_vault();
-            let sec_idx = vault.secret_generate(attrs).unwrap();
-            check_key_id_computation(vault, sec_idx);
+            let sec_idx = vault.secret_generate(attrs).await.unwrap();
+            check_key_id_computation(vault, sec_idx).await;
         }
     }
 
-    #[test]
-    fn secret_import_compute_key_id() {
+    #[tokio::test]
+    async fn secret_import_compute_key_id() {
         for attrs in flat_map_options(vec![new_curve255519_attrs(), new_bls_attrs()]) {
             let mut vault = new_vault();
-            let sec_idx = vault.secret_generate(attrs).unwrap();
-            let secret = vault.secret_export(&sec_idx).unwrap();
+            let sec_idx = vault.secret_generate(attrs).await.unwrap();
+            let secret = vault.secret_export(&sec_idx).await.unwrap();
             drop(vault); // The first vault was only used to generate random keys
 
             let mut vault = new_vault();
-            let sec_idx = vault.secret_import(secret.as_ref(), attrs).unwrap();
+            let sec_idx = vault.secret_import(secret.as_ref(), attrs).await.unwrap();
 
-            check_key_id_computation(vault, sec_idx);
+            check_key_id_computation(vault, sec_idx).await;
         }
     }
 
-    fn import_key(vault: &mut SoftwareVault, bytes: &[u8], attrs: SecretAttributes) -> KeyId {
-        let sec_idx = vault.secret_import(bytes, attrs).unwrap();
-        let public_key = vault.secret_public_key_get(&sec_idx).unwrap();
-        vault.compute_key_id_for_public_key(&public_key).unwrap()
+    async fn import_key(vault: &mut SoftwareVault, bytes: &[u8], attrs: SecretAttributes) -> KeyId {
+        let sec_idx = vault.secret_import(bytes, attrs).await.unwrap();
+        let public_key = vault.secret_public_key_get(&sec_idx).await.unwrap();
+        vault
+            .compute_key_id_for_public_key(&public_key)
+            .await
+            .unwrap()
     }
 
-    #[test]
-    fn secret_import_compute_key_id_predefined() {
+    #[tokio::test]
+    async fn secret_import_compute_key_id_predefined() {
         let bytes_c25519 = &[
             0x48, 0x95, 0x73, 0xcf, 0x4a, 0xe9, 0x16, 0x68, 0x86, 0x49, 0x8d, 0x3d, 0xd0, 0xde,
             0x00, 0x61, 0xb4, 0x01, 0xc1, 0xbf, 0x39, 0xd0, 0x8b, 0x7e, 0x4b, 0xf0, 0xa4, 0x90,
@@ -296,7 +313,7 @@ mod tests {
         ];
         let attrs = new_curve255519_attrs().unwrap();
         let mut vault = new_vault();
-        let key_id = import_key(&mut vault, bytes_c25519, attrs);
+        let key_id = import_key(&mut vault, bytes_c25519, attrs).await;
         assert_eq!(
             "f0e6821043434a9353e6c213a098f6d75ac916b23b3632c7c4c9c6d2e1fa1cf8",
             &key_id
@@ -311,7 +328,7 @@ mod tests {
                 ];
                 let attrs = new_bls_attrs().unwrap();
                 let mut vault = new_vault();
-                let key_id = import_key(&mut vault, bytes_bls, attrs);
+                let key_id = import_key(&mut vault, bytes_bls, attrs).await;
                 assert_eq!(
                     "604b7cf225a832c8fa822792dc7c484f5c49fb7a70ce87f1636b294ba7dbdc7b",
                     &key_id
